/*
 * Copyright 2003 - 2009 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 3751 $
 * Last Changed:    $Date: 2010-02-19 15:45:34 -0500 (Fri, 19 Feb 2010) $
 * Last Changed By: $Author: jan.moxter $
 */

package org.efaps.ui.wicket.components.table;

import java.util.Iterator;

import org.apache.wicket.Page;
import org.apache.wicket.behavior.SimpleAttributeModifier;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.MarkupStream;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.efaps.admin.dbproperty.DBProperties;
import org.efaps.ui.wicket.behaviors.SetSelectedRowBehavior;
import org.efaps.ui.wicket.components.table.row.RowPanel;
import org.efaps.ui.wicket.models.RowModel;
import org.efaps.ui.wicket.models.objects.UIFieldTable;
import org.efaps.ui.wicket.models.objects.UIRow;
import org.efaps.ui.wicket.models.objects.UITable;
import org.efaps.ui.wicket.pages.contentcontainer.ContentContainerPage;
import org.efaps.ui.wicket.resources.EFapsContentReference;
import org.efaps.ui.wicket.resources.StaticHeaderContributor;

/**
 * Class renders a table.
 *
 * @author The eFaps Team
 * @version $Id: TablePanel.java 3751 2010-02-19 20:45:34Z jan.moxter $
 */
public class TablePanel extends Panel
{
    /**
     * Reference to the style sheet.
     */
    public static final EFapsContentReference CSS = new EFapsContentReference(TablePanel.class, "TablePanel.css");

    /**
     * Needed for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * @param _wicketId wicket id of this component
     * @param _uitable    model for this component
     * @param _page     page this component is in
     */
    public TablePanel(final String _wicketId, final IModel<UITable> _uitable, final Page _page)
    {
        super(_wicketId, _uitable);

        final UITable uiTable = (UITable) super.getDefaultModelObject();

        if (!uiTable.isInitialized()) {
            uiTable.execute();
        }
        setOutputMarkupId(true);
        this.add(new SimpleAttributeModifier("class", "eFapsTableBody"));

        add(StaticHeaderContributor.forCss(TablePanel.CSS));

        final RepeatingView rowsRepeater = new RepeatingView("rowRepeater");
        add(rowsRepeater);

        if (uiTable.getValues().isEmpty()) {
            String text;
            if (uiTable.isFiltered()) {
                text = DBProperties.getProperty("WebTable.NoDataWithFilter");
            } else {
                text = DBProperties.getProperty("WebTable.NoData");
            }
            final Label nodata = new Label(rowsRepeater.newChildId(), text);
            nodata.add(new SimpleAttributeModifier("class", "eFapsTableNoData"));
            rowsRepeater.add(nodata);
        } else {
            boolean odd = true;
            for (final Iterator<UIRow> rowIter = uiTable.getValues().iterator(); rowIter.hasNext(); odd = !odd) {

                final RowPanel row = new RowPanel(rowsRepeater.newChildId(), new RowModel(rowIter.next()), this,
                                               ContentContainerPage.IFRAME_PAGEMAP_NAME.equals(_page.getPageMapName()));
                row.setOutputMarkupId(true);
                if (odd) {
                    row.add(new SimpleAttributeModifier("class", "eFapsTableRowOdd"));
                } else {
                    row.add(new SimpleAttributeModifier("class", "eFapsTableRowEven"));
                }
                rowsRepeater.add(row);
            }
        }
        if (uiTable.isCreateMode() || uiTable.isEditMode()) {
            rowsRepeater.add(new AjaxAddRemoveRowPanel(rowsRepeater.newChildId(), _uitable, rowsRepeater));
            if (uiTable instanceof UIFieldTable)  {
                if (((UIFieldTable) uiTable).isFirstTable()) {
                    this.add(new RowSelectedInput("selected"));
                } else {
                    this.add(new WebComponent("selected").setVisible(false));
                }
            } else {
                this.add(new RowSelectedInput("selected"));
            }
        } else {
            this.add(new WebComponent("selected").setVisible(false));
        }
    }

    /**
     * Class renders two hidden inputs used to commit the actual selected
     * row and column.
     */
    public class RowSelectedInput extends WebComponent
    {

        /**
         * Needed for serialization.
         */
        private static final long serialVersionUID = 1L;

        /**
         * @param _wicketId wicket id for this component
         */
        public RowSelectedInput(final String _wicketId)
        {
            super(_wicketId);
        }

        /**
         * @see org.apache.wicket.Component#onComponentTagBody(org.apache.wicket.markup.MarkupStream, org.apache.wicket.markup.ComponentTag)
         * @param _markupStream MarkupStream
         * @param _tag          ComponentTag
         */
        @Override
        protected void onComponentTagBody(final MarkupStream _markupStream, final ComponentTag _tag)
        {
            final StringBuilder html = new StringBuilder();
            html.append("<input type=\"hidden\" name=\"").append(SetSelectedRowBehavior.INPUT_NAME).append("\"/>")
                .append("<input type=\"hidden\" name=\"").append(SetSelectedRowBehavior.INPUT_ROW).append("\"/>");
            replaceComponentTagBody(_markupStream, _tag, html);
        }

    }
}
